/// <reference types="node" />
/**
 * Table 8: Interpretation of valid BITPIX value.
 * Value Data represented
 * 8 Character or unsigned binary integer
 * 16 16-bit two’s complement binary integer
 * 32 32-bit two’s complement binary integer
 * 64 64-bit two’s complement binary integer
 * −32 IEEE single-precision floating point
 * −64 IEEE double-precision floating point
 */
declare enum PixelDataType {
    UnsignedBinary = "8",
    Signed16 = "16",
    Signed32 = "32",
    Signed64 = "64",
    SingleFloat = "-32",
    DoubleFloat = "-64"
}
/**
 * 3.2. Individual FITS Structures
 * The primary HDU and every extension HDU shall consist of
 * one or more 2880-byte header blocks immediately followed by
 * an optional sequence of associated 2880-byte data blocks. The
 * header blocks shall contain only the restricted set of ASCII-text
 * characters, decimal 32 through 126 (hexadecimal 20 through
 * 7E). The ASCII control characters with decimal values less than
 * 32 (including the null, tab, carriage return, and line-feed characters), and the delete character (decimal 127 or hexadecimal 7F)
 * must not appear anywhere within a header block.
 */
export declare class FitsStructure {
    static readonly BLOCK_SIZE = 2880;
    static readonly RECORD_SIZE = 80;
    static numBlocks(numBytes: number): number;
    static getDataValue(dataType: PixelDataType, buff: Buffer, offset: number): number;
    /**
     * A mapping of keyword names mapped to their values.
     */
    private readonly headers;
    /**
     * Raw values of the primary header.
     */
    private rawHeaders;
    private readonly commentaryHeaders;
    private readonly rawData;
    constructor(data: Buffer);
    getNextStructure(): FitsStructure | undefined;
    getDataBlock(blockNum: number): Buffer;
    /**
     * @returns An array of entries in the primary header as they appear in the file.
     */
    getRawHeaders(): string[];
    getHeaderMap(): Map<string, string>;
    getNumHeaderBlocks(): number;
    getNumDataBlocks(): number;
    getMetadataValue(key: string): string;
    getMetadataBlock(blockNum: number): string[];
    numAxis(): number;
    sizeAxis(index: number): number;
    centerPixel(): {
        x: number;
        y: number;
    } | undefined;
    getDataUnit(): Buffer;
    /**
     * @returns 1D array of all data values
     */
    getDataValues(): Array<number>;
    /**
     * TODO handle n axis.
     * @returns 2D array of all data values.
     */
    getDataValuesArray(): Array<Array<number>>;
    getDataSize(): number;
    getNumStructureBlocks(): number;
    getNumStructureBytes(): number;
    pixelDataType(): PixelDataType;
    isImage(): boolean | "";
    /**
     * BITPIX keyword. The value field shall contain an integer. The
     * absolute value is used in computing the sizes of data structures.
     * It shall specify the number of bits that represent a data value in
     * the associated data array. The only valid values of BITPIX are
     * given in Table 8. Writers of FITS arrays should select a BITPIX
     * data type appropriate to the form, range of values, and accuracy
     * of the data in the array
     *
     * Table 8: Interpretation of valid BITPIX value.
     * Value Data represented
     * 8 Character or unsigned binary integer
     * 16 16-bit two’s complement binary integer
     * 32 32-bit two’s complement binary integer
     * 64 64-bit two’s complement binary integer
     * −32 IEEE single-precision floating point
     * −64 IEEE double-precision floating point
     */
    bitsPerPixel(): number;
    getDataStats(): {
        min: number;
        max: number;
        median: number;
        mean: number;
        stdDev: number;
    };
}
export {};
